-- Keyboard driver
local array = require 'array'

local keychord = {}
local I = {}

local Modifiers = {'lctrl', 'rctrl', 'lalt', 'ralt', 'lshift', 'rshift', 'lgui', 'rgui'}

function keychord.collect_keypress(key, scancode, isrepeat)
  if array.find(Modifiers, key) then
    -- do nothing when the modifier is pressed
    return
  end
  -- include the modifier(s) when the non-modifer is pressed
  return I.combine_modifiers(key)
end

function I.combine_modifiers(key)
  if love.keyboard.isModifierActive then  -- waiting for LÖVE v12
    if key:match('^kp') then
      key = I.translate_numlock(key)
    end
  end
  local result = ''
  if keychord.ctrl_down() then
    result = result..'C-'
  end
  if keychord.alt_down() then
    result = result..'M-'
  end
  if keychord.shift_down() then
    result = result..'S-'  -- don't try to use this with letters/digits
  end
  if keychord.cmd_down() then
    result = result..'s-'
  end
  result = result..key
  return result
end

function keychord.any_modifier_down()
  return keychord.ctrl_down() or keychord.alt_down() or keychord.shift_down() or keychord.cmd_down()
end

function keychord.ctrl_down()
  return key_down('lctrl') or key_down('rctrl')
end

function keychord.alt_down()
  return key_down('lalt') or key_down('ralt')
end

function keychord.shift_down()
  return key_down('lshift') or key_down('rshift')
end

function keychord.cmd_down()
  return key_down('lgui') or key_down('rgui')
end

function keychord.is_cursor_movement(key)
  return array.find({'left', 'right', 'up', 'down', 'home', 'end', 'pageup', 'pagedown'}, key)
end

-- mappings only to non-printable keys; leave out mappings that textinput will handle
local Numlock_off = {
  kp0='insert',
  kp1='end',
  kp2='down',
  kp3='pagedown',
  kp4='left',
  -- numpad 5 translates to nothing
  kp6='right',
  kp7='home',
  kp8='up',
  kp9='pageup',
  ['kp.']='delete',
  -- LÖVE handles keypad operators in textinput
  -- what's with the `kp=` and `kp,` keys? None of my keyboards have one.
  -- Hopefully LÖVE handles them as well in textinput.
  kpenter='enter',
  kpdel='delete',
}
local Numlock_on = {
  kpenter='enter',
  kpdel='delete',
}
function I.translate_numlock(key)
  if love.keyboard.isModifierActive('numlock') then
    return Numlock_on[key] or key
  else
    return Numlock_off[key] or key
  end
  return key
end

return keychord
